package engine;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

/**
 * Classe de capture des commandes (clavier/souris) - singleton.
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0
 */

public class Commands implements KeyListener, MouseListener, MouseMotionListener {

	public static final int UP = 90;
	public static final int DOWN = 83;
	public static final int LEFT = 81;
	public static final int RIGHT = 68;
	public static final int QUIT = 27;

	private static Commands instance = null;
	private boolean updateMouse = false;
	private boolean clicked = false;

	/**
	 * liste des touches actives
	 */
	public volatile List<Integer> keys = new ArrayList<Integer>();

	public int mx, my, cmx, cmy;

	private Commands() {
	}

	public boolean isMousePressed() {
		return updateMouse;
	}

	public boolean isMouseClicked() {
		if (clicked) {
			clicked = false;
			return true;
		}
		return false;
	}

	public final synchronized static Commands getInstance() {
		if (instance == null)
			instance = new Commands();
		return instance;
	}

	@Override
	public void keyTyped(KeyEvent e) {
	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (!keys.contains(e.getKeyCode()))
			keys.add(e.getKeyCode());
	}

	@Override
	public void keyReleased(KeyEvent e) {
		int index = keys.indexOf(e.getKeyCode());
		if (index >= 0)
			keys.remove(index);
	}

	// mouse management

	@Override
	public void mouseClicked(MouseEvent arg0) {
		// on utilise pas cette methode car elle ne s'active pas en cas de drag.

	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		updateMouse = true;
		arg0.consume();
		// gestion du simple clic
		clicked = true;
		cmx = mx;
		cmy = my;
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		updateMouse = false;
		arg0.consume();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent )
	 */
	@Override
	public void mouseDragged(MouseEvent arg0) {
		mx = arg0.getX();
		my = arg0.getY();
		arg0.consume();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseMoved(MouseEvent arg0) {
		mx = arg0.getX();
		my = arg0.getY();
		arg0.consume();
	}

}
