package engine;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.WindowConstants;

import game.Const;
import tools.generated.Textures;
//TODO idealement le moteur ne devrais pas avoir de ref sur les textures ni sur const

public class Render {

	public static final int BotLAYER = 0;
	public static final int MidLAYER = 1;
	public static final int TopLAYER = 2;
	public static final int LightLAYER = 3;
	public static final int UILAYER = 4;
	private static final int layerslen = 5;

	private static final boolean fullscreen = false;
	// TODO revoir la projection des coordonées

	private final JFrame frame = new JFrame();
	private final Commands commands;
	private final Graphics2D frame_g, buffer_g;
	private final BufferedImage buffer;
	int screenWidth, screenHeight;

	private final Layer[] layers = new Layer[layerslen];
	private final List<IRenderable> renderables = new ArrayList<>();
	private static Render instance = null;

	public final synchronized static Render getInstance() {
		if (instance == null)
			instance = new Render();
		return instance;
	}

	// initialisation du moteur de jeu
	private Render() {
		if (fullscreen) {

			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			screenWidth = (int) screenSize.getWidth();
			screenHeight = (int) screenSize.getHeight();
		} else {
			screenWidth = Const.W;
			screenHeight = Const.H;
		}
		commands = Commands.getInstance();
		frame.setSize(screenWidth, screenHeight);
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		// if(fullscreen)
		frame.setUndecorated(true);
		frame.setResizable(false);
		frame.addKeyListener(commands);
		frame.addMouseListener(commands);
		frame.addMouseMotionListener(commands);
		frame.setIgnoreRepaint(true);
		frame.setLocationRelativeTo(frame.getParent());
		frame.setVisible(true);

		// construction de la liste de rendu
		for (int i = 0; i < layers.length; i++) {
			layers[i] = new Layer();
			renderables.add(layers[i]);
		}

		// initialisation de l'affichage :
		buffer = new BufferedImage(Const.W, Const.H, BufferedImage.TYPE_INT_ARGB);
		frame_g = (Graphics2D) frame.getGraphics();
		buffer_g = (Graphics2D) buffer.getGraphics();
		buffer_g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
	}

	/**
	 * recupere le calque d'id correspondant (null sinon)
	 * 
	 * @param id
	 * @return le layer demandé
	 */
	public Layer getLayer(int id) {
		if (id >= 0 && id < layers.length)
			return layers[id];
		return null;
	}

	/**
	 * reinitialise les layers
	 */
	public void resetLayers() {
		for (int i = 0; i < layers.length; i++)
			layers[i].clear();
	}

	/**
	 * demande la mise a jour de l'affichage
	 */
	public void update() {
		// on dessine dans le buffer
		buffer_g.drawImage(Textures.screen_background, 0, 0, frame);
		for (IRenderable r : renderables)
			r.render(buffer_g);
		// on rafraichie
		frame_g.drawImage(buffer, 0, 0, screenWidth, screenHeight, frame);
	}

	/**
	 * methode permettant de remplire un cercle de rayon r, centré en x,y dans g
	 * 
	 * @param g
	 * @param x
	 * @param y
	 * @param r
	 */
	public static final void drawCircle(Graphics2D g, int x, int y, int r) {
		x = x - (r);
		y = y - (r);
		g.fillOval(x, y, 2 * r, 2 * r);
	}
}
