package game.objects;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import engine.Render;
import game.Const;

public abstract class Tower implements ITower {

	private final int x, y, range, sqrange, cx, cy, round, power, price;

	private final BufferedImage texture;

	public Tower(int x, int y, int range, int round, int power, int price, BufferedImage texture) {
		this.x = x - texture.getWidth() / 2;
		this.y = y - texture.getHeight() / 2;
		this.range = range;
		this.sqrange = range * range;
		this.cx = x;
		this.cy = y;
		this.round = round;
		this.power = power;
		this.price = price;
		this.texture = texture;
	}

	@Override
	public void render(Graphics2D buffer) {
		buffer.drawImage(texture, x, y, null);
		if (Const.DEBUG) {
			buffer.setColor(Const.MASQUE);
			Render.drawCircle(buffer, cx, cy, range);
		}
	}

	@Override
	public int round() {
		return round;

	}

	@Override
	public int power() {
		return power;

	}

	@Override
	public int getCost() {
		return price;
	}

	@Override
	public final boolean isInRange(int ox, int oy) {
		return (cx - ox) * (cx - ox) + (cy - oy) * (cy - oy) < sqrange;
	}
}
