package game.renderables;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import game.Const;
import tools.generated.Textures;
import engine.IRenderable;

/**
 * affichage de la couche visuelle d'interface
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0
 */

public class UI implements IRenderable {

	private Font defaultFont = null;
	private Font bigFont = null;

	private int cash = 0;
	private int invasion = 0;
	private int nocashTimer = 0;

	public UI() {
	}

	public void update(int current_cash, int current_inv) {
		cash = current_cash;
		if (current_inv <= Const.MAX_INVA)
			invasion = current_inv;
	}

	public void nocashBlink() {
		nocashTimer = 20;
	}

	@Override
	public void render(Graphics2D image) {
		if (defaultFont == null) {
			defaultFont = image.getFont();
			bigFont = defaultFont.deriveFont(Font.BOLD, 16);
		}

		image.drawImage(Textures.screen_ui, 0, 0, null);
		image.setColor(Color.BLACK);
		image.drawString("The WALL v" + Const.VERSION, 1180, 38);

		// status de l'invasion
		image.setColor(Color.RED);
		image.fillRect(351, 47, invasion, 14);

		// cash restant, annimation si l'utilisateur tente de l'utiliser mais pas assez
		if (nocashTimer > 0) {
			nocashTimer--;
			image.setColor(Const.NOCASH);
			if (nocashTimer % 10 < 5)
				image.fillRect(690, 38, 200, 32);
		}

		image.setFont(bigFont);
		image.setColor(Color.GRAY);
		image.drawString("Status de l'invasion ...", 335, 84);
		image.setColor(Color.YELLOW);
		image.drawString("Tresorerie : " + cash + " $", 696, 61);
		image.setColor(Color.GRAY);
		image.drawString("Tresorerie : " + cash + " $", 695, 60);

		image.setColor(Color.BLACK);

		image.setFont(defaultFont);

		// bouttons
		for (int i = 0; i < Const.btList.length; i++) {
			Const.btList[i].render(image, Const.BT_MARGIN, Const.BT_OFFSET + i * Const.BT_GAP);
		}

		// Dessin de la carte du monde
		for (int i = 0; i < Const.map[0].length; i++)
			image.drawImage(Const.map[mapPanic[i]][i], 0, 0, null);

		// zone de placement de tourelle
		if (Const.DEBUG) {
			image.setColor(Const.MASQUE);
			image.drawRect(Const.AREA_LEFT, Const.AREA_TOP, 1070, Const.SPAWN_RANGE + 32);
		}
	}

	private int[] mapPanic = { 0, 0, 0, 0, 0, 0 };
	private int spended = 0;

	public boolean checkmap() {
		int sum = 0;
		for (int crise : mapPanic)
			sum += crise;
		return sum >= 12;
	}

	/** gestion de la panique : augmentation */
	public void panic(int price) {
		spended += price;

		if (spended > Const.PANIC) {
			int idx = (int) (Math.random() * 6);
			for (int i = 0; i < 10; i++) {
				if (mapPanic[idx] < 2)
					break;
				idx = (int) (Math.random() * 6);
			}
			if (mapPanic[idx] < 2) {
				mapPanic[idx] = mapPanic[idx] + 1;
				spended = 0;
			}
		}
	}

	/** gestion de la panique : reduction */
	public void calm() {
		int max = 0;
		int idx = 0;
		for (int i = 0; i < mapPanic.length; i++) {
			if (mapPanic[i] > max) {
				max = mapPanic[i];
				idx = i;
			}
		}
		if (mapPanic[idx] > 0)
			mapPanic[idx] = mapPanic[idx] - 1;
	}

}
