package tools.generators;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import tools.Tools;
import tools.generators.core.Generator;
import tools.generators.core.JGenerationTools;

/**
 * Generateur de la classe de chargement d'image.
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0
 * @since
 * 
 */
public class PicturesLoaderGenerator implements Generator {

	private final static String ls = System.lineSeparator();

	private final static String[] tokenList = { "_DATE_", "tools.generators.templates", "_IMGCNT_", "DECLARATION",
			"LOADING_ROUTINE" };
	private static final int DATE = 0;
	private static final int PACKAGE = 1;
	private static final int IMGCNT = 2;
	private static final int DECLARATION = 3;
	private static final int ROUTINE = 4;

	private static List<File> piclist = null;
	private static Tools t = null;

	private static JGenerationTools gt = null;
	private static PicturesLoaderGenerator rlg = null;
	private static String subpath = null;

	public static void main(String[] args) throws IOException, InterruptedException {

		// on initialise les propriete
		ConfigGenerator.ToolsInit();
		rlg = new PicturesLoaderGenerator();
		gt = new JGenerationTools();
		t = Tools.getInstance();
		t.log("-------- -------- [running PicturesLoaderGenerator]");
		// on genere les fichiers d'images
		generate("Textures", "Textures.java");

	}

	private static void generate(String relativePath, String className) throws IOException {
		// preparation des données (images)
		piclist = new ArrayList<File>();
		subpath = relativePath + "/";
		t.getFilesIn(t.getStringFor("PICTURE_PATH") + relativePath, piclist);
		// on fait un test rapide pour les espaces dans le nom de fichier
		ArrayList<File> removeList = new ArrayList<File>();
		for (File pic : piclist) {
			if (pic.getName().contains(" ")) {
				t.logError("nom invalide (pas d'espace dans le nom : " + pic.getName()
						+ ") -> image non prise en charge.");
				removeList.add(pic);
			}
		}
		// on vide les element indesirable
		for (File pic : removeList)
			piclist.remove(pic);
		// on genere le chargeur d'image
		gt.init("./src/tools/generated/" + className, "./src/tools/generators/templates/PicturesLoaderTemplate.java",
				tokenList, false);
		gt.StartGeneration(rlg);
		gt.close();
	}

	@Override
	public String replaceLine(int token) {
		StringBuilder line = new StringBuilder();
		t.log("TOKEN: " + tokenList[token]);
		for (File pic : piclist) {
			if (token == DECLARATION) {
				// Ici c'est un peu bourrin, on considere que le fichier n'a pas
				// de . dans son nom. En effet en java les variable ne suportent
				// pas ces caracteres
				line.append("public static BufferedImage ");
				line.append(pic.getName().substring(0, pic.getName().indexOf('.')));
				line.append(";");
				line.append(ls);
			} else if (token == ROUTINE) {
				line.append("f = new File(path +\"" + subpath + "\"+ \"");
				line.append(pic.getName());
				line.append("\");");
				line.append(ls);
				line.append(pic.getName().substring(0, pic.getName().indexOf('.')));
				line.append(" = ImageIO.read(f);");
				line.append(ls);
				line.append("state++;");
				line.append(ls);
				t.log(pic.getName() + "...");
			}

		}
		return line.toString();
	}

	@Override
	public String replaceTokens(String base, int token) {
		t.log("TOKEN: " + tokenList[token]);
		String line = "// " + tokenList[token]
				+ ": Wrong token use (Template position and Generator usage are inconsistent)";
		if (token == DATE)
			line = base.replaceAll(tokenList[token], t.date2String("en"));
		else if (token == PACKAGE)
			line = base.replaceAll(tokenList[token], "tools.generated");
		else if (token == IMGCNT)
			line = base.replaceAll(tokenList[token], Integer.toString(piclist.size()));
		else
			t.logError(line);
		return line;
	}
}
